/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2016  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#ifndef ENCODERCHANNELHEADERFILE
#define ENCODERCHANNELHEADERFILE

#include <vector>

#include "../../core/vector2.h"

class TEncoderChannel
{
public:

enum TChannelMode
{
	MODE_SINE=0,
	MODE_COSINE
};
	
// CREATORS
	TEncoderChannel(void);
	~TEncoderChannel(void);
	
// ACCESSORS
	double Gain(void) const {return d_gain;}
	double Offset(void) const {return d_offset;}
	double Phase(void) const {return d_phase;}
	
	double Peak_Positive_Level(void) const;
	double Peak_Negative_Level(void) const;
	
	std::vector<TVector2> Signal_Data(void) const {return d_signal_data;}
	double Signal_Data(const double &angle) const;
	
// MANIPULATORS
	void Set_Mode(TChannelMode mode) {d_channel_mode = mode;}
		
	void Set_Gain(const double &gain);
	void Set_Offset(const double &offset);
	void Set_Phase(const double &phase);
	
	void Set_Point_Count(const int count) {d_point_count = count;}
	void Update_Signals(void);
	
private:
	int									d_point_count;
	double								d_gain;
	double								d_offset;
	double								d_phase;
	std::vector<TVector2>				d_signal_data;
	
	TChannelMode						d_channel_mode;
	
// NOT IMPLEMENTED
	TEncoderChannel(const TEncoderChannel&);
	TEncoderChannel& operator=(const TEncoderChannel&);
	
};

#endif