/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2016  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#include <QApplication>
#include <QComboBox>
#include <QDoubleSpinBox>
#include <QFont>
#include <QFrame>
#include <QGridLayout>
#include <QGroupBox>
#include <QHBoxLayout>
#include <QLabel>
#include <QPlainTextEdit>
#include <QPushButton>
#include <QToolButton>
#include <QSizePolicy>
#include <QSpacerItem>
#include <QStringList>
#include <QTabWidget>
#include <QVBoxLayout>
#include <QIcon>
#include <QScrollBar>
#include <QWidget>

#include "../../core/graphwidget.h"

#include "encodersimulation.h"

TEncoderSimulation::TEncoderSimulation(
	const QWidget						*parent,
	const Qt::WindowFlags				flags)
:QDialog(const_cast<QWidget*>(parent),flags)
{
	QFont								text_font;
	QFont								footer_font;
	QFont								status_font;
	QFrame								*channel_hline;
	QFrame								*channel_vline;
	QFrame								*channela_hline;
	QFrame								*channelb_hline;
	QFrame								*line;
	QGridLayout							*channel_layout;
	QGridLayout							*setup_layout;
	QGroupBox							*output_group;
	QGroupBox							*setup_group;
	QHBoxLayout							*button_hlayout;
	QHBoxLayout							*graph_button_hlayout;
	QLabel								*footer_label;
	QLabel								*setup_channela_label;
	QLabel								*setup_channelb_label;
	QLabel								*setup_gaina_label;
	QLabel								*setup_gainb_label;
	QLabel								*setup_offseta_label;
	QLabel								*setup_offsetb_label;
	QLabel								*setup_phasea_label;
	QLabel								*setup_phaseb_label;
	QLabel								*setup_pitch_label;
	QLabel								*setup_pitch_unit_label;
	QLabel								*setup_resolution_label;
	QLabel								*setup_resolution_unit_label;
	QLabel								*display_mode_label;
	QPushButton							*close_button;
	QSizePolicy							size_policy(QSizePolicy::MinimumExpanding,QSizePolicy::Preferred);
	QSpacerItem							*button_hspacer;
	QSpacerItem							*graph_button_hspacer;
	QVBoxLayout							*output_vlayout;
	QVBoxLayout							*output_graph_vlayout;
	QVBoxLayout							*output_text_vlayout;
	QVBoxLayout							*dialog_vlayout;
	QWidget								*graphic_tab_widget;
	QWidget								*text_tab_widget;

	size_policy.setHorizontalStretch(0);
	size_policy.setVerticalStretch(0);

	text_font.setFamily(QStringLiteral("Courier New"));
	text_font.setPointSize(10);

	footer_font.setFamily(QStringLiteral("Verdana"));
	footer_font.setPointSize(8);
	footer_font.setItalic(true);
	
	status_font.setPointSize(12);
	status_font.setBold(true);

	this->resize(801,655);

	dialog_vlayout = new QVBoxLayout(this);

	setup_group = new QGroupBox(this);

	setup_layout = new QGridLayout(setup_group);
	
	graph_button_hlayout = new QHBoxLayout();
	
	graph_button_hspacer = new QSpacerItem(0,0,QSizePolicy::Expanding,QSizePolicy::Minimum);
	graph_button_hlayout->addItem(graph_button_hspacer);
	
	display_mode_label = new QLabel(this);
	graph_button_hlayout->addWidget(display_mode_label);
	
	d_graph_mode_button = new QToolButton(this);
	d_graph_mode_button->setFixedSize(22,22);
	d_graph_mode_button->setCheckable(true);
	graph_button_hlayout->addWidget(d_graph_mode_button);

	setup_layout->addLayout(graph_button_hlayout,0,3,1,1);

	d_signal_graph_widget = new TGraphWidget(setup_group);
	d_signal_graph_widget->setSizePolicy(size_policy);
	d_signal_graph_widget->SetScaleMinimumY(1.0);
	setup_layout->addWidget(d_signal_graph_widget,1,3,2,1);

	setup_resolution_label = new QLabel(setup_group);
	setup_layout->addWidget(setup_resolution_label,1,0,1,1);

	setup_pitch_label = new QLabel(setup_group);
	setup_layout->addWidget(setup_pitch_label,0,0,1,1);

	d_setup_resolution_combo = new QComboBox(setup_group);
	d_setup_resolution_combo->insertItems(0,QStringList()
			<< QStringLiteral("0.0100")
			<< QStringLiteral("0.0050")
			<< QStringLiteral("0.0010")
			<< QStringLiteral("0.0005")
			<< QStringLiteral("0.0001"));
	
	setup_layout->addWidget(d_setup_resolution_combo,1,1,1,1);

	d_setup_pitch_combo = new QComboBox(setup_group);
	d_setup_pitch_combo->insertItems(0,QStringList()
			<< QStringLiteral("0.120")
			<< QStringLiteral("0.100")
			<< QStringLiteral("0.080")
			<< QStringLiteral("0.040")
			<< QStringLiteral("0.020"));
	
	setup_layout->addWidget(d_setup_pitch_combo,0,1,1,1);

	setup_pitch_unit_label = new QLabel(setup_group);
	setup_layout->addWidget(setup_pitch_unit_label,0,2,1,1);

	channel_layout = new QGridLayout();

	channel_hline = new QFrame(setup_group);
	channel_hline->setFrameShape(QFrame::HLine);
	channel_hline->setFrameShadow(QFrame::Sunken);
	channel_layout->addWidget(channel_hline,0,0,1,5);

	setup_channela_label = new QLabel(setup_group);
	setup_channela_label->setAlignment(Qt::AlignCenter);
	channel_layout->addWidget(setup_channela_label,1,0,1,2);

	channel_vline = new QFrame(setup_group);
	channel_vline->setFrameShape(QFrame::VLine);
	channel_vline->setFrameShadow(QFrame::Sunken);
	channel_layout->addWidget(channel_vline,1,2,5,1);

	setup_channelb_label = new QLabel(setup_group);
	setup_channelb_label->setAlignment(Qt::AlignCenter);
	channel_layout->addWidget(setup_channelb_label,1,3,1,2);

	channela_hline = new QFrame(setup_group);
	channela_hline->setFrameShape(QFrame::HLine);
	channela_hline->setFrameShadow(QFrame::Sunken);
	channel_layout->addWidget(channela_hline,2,0,1,2);

	channelb_hline = new QFrame(setup_group);
	channelb_hline->setFrameShape(QFrame::HLine);
	channelb_hline->setFrameShadow(QFrame::Sunken);
	channel_layout->addWidget(channelb_hline,2,3,1,2);

	setup_gaina_label = new QLabel(setup_group);
	setup_gaina_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	channel_layout->addWidget(setup_gaina_label,3,0,1,1);

	d_setup_gaina_spin = new QDoubleSpinBox(setup_group);
	d_setup_gaina_spin->setAlignment(Qt::AlignCenter);
	d_setup_gaina_spin->setMinimum(0.1);
	d_setup_gaina_spin->setMaximum(2);
	d_setup_gaina_spin->setSingleStep(0.05);
	d_setup_gaina_spin->setValue(1);
	channel_layout->addWidget(d_setup_gaina_spin,3,1,1,1);

	setup_gainb_label = new QLabel(setup_group);
	setup_gainb_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	channel_layout->addWidget(setup_gainb_label,3,3,1,1);

	d_setup_gainb_spin = new QDoubleSpinBox(setup_group);
	d_setup_gainb_spin->setAlignment(Qt::AlignCenter);
	d_setup_gainb_spin->setMinimum(0.1);
	d_setup_gainb_spin->setMaximum(2);
	d_setup_gainb_spin->setSingleStep(0.05);
	d_setup_gainb_spin->setValue(1);
	channel_layout->addWidget(d_setup_gainb_spin,3,4,1,1);

	setup_offseta_label = new QLabel(setup_group);
	setup_offseta_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	channel_layout->addWidget(setup_offseta_label,4,0,1,1);

	d_setup_offseta_spin = new QDoubleSpinBox(setup_group);
	d_setup_offseta_spin->setAlignment(Qt::AlignCenter);
	d_setup_offseta_spin->setMinimum(-1);
	d_setup_offseta_spin->setMaximum(1);
	d_setup_offseta_spin->setSingleStep(0.1);
	channel_layout->addWidget(d_setup_offseta_spin,4,1,1,1);

	setup_offsetb_label = new QLabel(setup_group);
	setup_offsetb_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	channel_layout->addWidget(setup_offsetb_label,4,3,1,1);

	d_setup_offsetb_spin = new QDoubleSpinBox(setup_group);
	d_setup_offsetb_spin->setAlignment(Qt::AlignCenter);
	d_setup_offsetb_spin->setMinimum(-1);
	d_setup_offsetb_spin->setMaximum(1);
	d_setup_offsetb_spin->setSingleStep(0.1);
	channel_layout->addWidget(d_setup_offsetb_spin,4,4,1,1);

	setup_phasea_label = new QLabel(setup_group);
	setup_phasea_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	channel_layout->addWidget(setup_phasea_label,5,0,1,1);

	d_setup_phasea_spin = new QDoubleSpinBox(setup_group);
	d_setup_phasea_spin->setAlignment(Qt::AlignCenter);
	d_setup_phasea_spin->setDecimals(6);
	d_setup_phasea_spin->setMinimum(-0.78539816339745);
	d_setup_phasea_spin->setMaximum(0.78539816339745);
	d_setup_phasea_spin->setSingleStep(0.07853981633975);
	channel_layout->addWidget(d_setup_phasea_spin,5,1,1,1);

	setup_phaseb_label = new QLabel(setup_group);
	setup_phaseb_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	channel_layout->addWidget(setup_phaseb_label,5,3,1,1);

	d_setup_phaseb_spin = new QDoubleSpinBox(setup_group);
	d_setup_phaseb_spin->setAlignment(Qt::AlignCenter);
	d_setup_phaseb_spin->setDecimals(6);
	d_setup_phaseb_spin->setMinimum(-0.78539816339745);
	d_setup_phaseb_spin->setMaximum(0.78539816339745);
	d_setup_phaseb_spin->setSingleStep(0.07853981633975);
	d_setup_phaseb_spin->setValue(0);
	channel_layout->addWidget(d_setup_phaseb_spin,5,4,1,1);
	setup_layout->addLayout(channel_layout,2,0,1,3);

	setup_resolution_unit_label = new QLabel(setup_group);
	setup_layout->addWidget(setup_resolution_unit_label,1,2,1,1);
	dialog_vlayout->addWidget(setup_group);
	
	d_output_status_label = new QLabel(this);
	d_output_status_label->setFont(status_font);
	dialog_vlayout->addWidget(d_output_status_label);

	output_group = new QGroupBox(this);

	output_vlayout = new QVBoxLayout(output_group);
	output_vlayout->setContentsMargins(0, 0, 0, 0);
	
	d_output_view_tab = new QTabWidget(output_group);

	graphic_tab_widget = new QWidget();
	d_output_view_tab->addTab(graphic_tab_widget,QStringLiteral("Graphic"));

	output_graph_vlayout = new QVBoxLayout(graphic_tab_widget);

	d_output_graph_widget = new TGraphWidget(graphic_tab_widget);
	d_output_graph_widget->SetScaleMinimumY(0.001);
	output_graph_vlayout->addWidget(d_output_graph_widget);

	text_tab_widget = new QWidget();
	d_output_view_tab->addTab(text_tab_widget,QStringLiteral("Text"));

	output_text_vlayout = new QVBoxLayout(text_tab_widget);

	d_output_text_edit = new QPlainTextEdit(text_tab_widget);
	d_output_text_edit->setFont(text_font);
	d_output_text_edit->setUndoRedoEnabled(false);
	d_output_text_edit->setLineWrapMode(QPlainTextEdit::NoWrap);
	d_output_text_edit->setReadOnly(true);
	output_text_vlayout->addWidget(d_output_text_edit);
	d_output_view_tab->setCurrentIndex(0);
	output_vlayout->addWidget(d_output_view_tab);
	dialog_vlayout->addWidget(output_group);

	line = new QFrame(this);
	line->setFrameShape(QFrame::HLine);
	line->setFrameShadow(QFrame::Sunken);
	dialog_vlayout->addWidget(line);

	button_hlayout = new QHBoxLayout();

	footer_label = new QLabel(this);
	footer_label->setFont(footer_font);
	button_hlayout->addWidget(footer_label);

	button_hspacer = new QSpacerItem(0,0,QSizePolicy::Expanding,QSizePolicy::Minimum);
	button_hlayout->addItem(button_hspacer);

	close_button = new QPushButton(this);
	close_button->setAutoDefault(false);
	button_hlayout->addWidget(close_button);
	dialog_vlayout->addLayout(button_hlayout);

	QWidget::setTabOrder(d_setup_pitch_combo,d_setup_resolution_combo);
	QWidget::setTabOrder(d_setup_resolution_combo,d_setup_gaina_spin);
	QWidget::setTabOrder(d_setup_gaina_spin,d_setup_offseta_spin);
	QWidget::setTabOrder(d_setup_offseta_spin,d_setup_phasea_spin);
	QWidget::setTabOrder(d_setup_phasea_spin,d_setup_gainb_spin);
	QWidget::setTabOrder(d_setup_gainb_spin,d_setup_offsetb_spin);
	QWidget::setTabOrder(d_setup_offsetb_spin,d_setup_phaseb_spin);
	QWidget::setTabOrder(d_setup_phaseb_spin,d_output_view_tab);
	QWidget::setTabOrder(d_output_view_tab,d_output_text_edit);
	QWidget::setTabOrder(d_output_text_edit,close_button);

	this->setWindowTitle(QStringLiteral("Encoder Simulation - 1.0"));
	
	setup_group->setTitle(QStringLiteral("Encoder Setup"));
	setup_resolution_label->setText(QStringLiteral("Interpolated Resolution:"));
	setup_pitch_label->setText(QStringLiteral("Scale Pitch:"));
	setup_pitch_unit_label->setText(QStringLiteral("mm"));
	setup_channela_label->setText(QStringLiteral("Channel A (SIN)"));
	setup_channelb_label->setText(QStringLiteral("Channel B (COS)"));
	setup_gaina_label->setText(QStringLiteral("Gain:"));
	setup_gainb_label->setText(QStringLiteral("Gain:"));
	setup_offseta_label->setText(QStringLiteral("Offset:"));
	setup_offsetb_label->setText(QStringLiteral("Offset:"));
	setup_phasea_label->setText(QStringLiteral("Phase:"));
	setup_phaseb_label->setText(QStringLiteral("Phase:"));
	setup_resolution_unit_label->setText(QStringLiteral("mm"));
	display_mode_label->setText(QStringLiteral("Display Mode:"));
	output_group->setTitle(QStringLiteral("Output Evaluation"));
	footer_label->setText(QStringLiteral("Encoder Simulation © 2016 Select Calibration Incorporated.  www.selectcalibration.ca"));
	close_button->setText(QStringLiteral("Close"));
	
	// default values
	d_output_graph_widget->VerticalLegend(QStringLiteral("Interpolation Error"));
	d_output_graph_widget->HorizontalLegend(QStringLiteral("Scale Position"));
	d_output_graph_widget->Title(QStringLiteral("Scale Interpolation Data"));

	d_signal_graph_widget->VerticalLegend(QStringLiteral("Signal"));
	d_signal_graph_widget->HorizontalLegend(QStringLiteral("Channel AB Radian Position"));
	d_signal_graph_widget->Title(QStringLiteral("Oscilloscope - Dual Channel Mode"));

	d_channel_a.Set_Mode(TEncoderChannel::MODE_SINE);
	d_channel_b.Set_Mode(TEncoderChannel::MODE_COSINE);
	d_setup_resolution_combo->setCurrentIndex(3);	// 0.0005 mm
	d_setup_pitch_combo->setCurrentIndex(1);		// 0.1000 mm
	
	d_graph_mode_button->setChecked(false);
	d_graph_mode_button->setIcon(QIcon(":/gui/gui_ty.png"));
	
	d_channel_a.Update_Signals();
	d_channel_b.Update_Signals();
	
	this->Update_Encoder_Signal();
	this->Update_Interpolator_Output();
	
	connect(close_button,SIGNAL(clicked(void)),this,SLOT(close(void)));
	
	connect(d_setup_gaina_spin,SIGNAL(valueChanged(double)),this,SLOT(Gain_A(const double)));
	connect(d_setup_gainb_spin,SIGNAL(valueChanged(double)),this,SLOT(Gain_B(const double)));
	connect(d_setup_offseta_spin,SIGNAL(valueChanged(double)),this,SLOT(Offset_A(const double)));
	connect(d_setup_offsetb_spin,SIGNAL(valueChanged(double)),this,SLOT(Offset_B(const double)));
	connect(d_setup_phasea_spin,SIGNAL(valueChanged(double)),this,SLOT(Phase_A(const double)));
	connect(d_setup_phaseb_spin,SIGNAL(valueChanged(double)),this,SLOT(Phase_B(const double)));
	
	connect(d_setup_resolution_combo,SIGNAL(currentIndexChanged(int)),this,SLOT(Setup_Changed(void)));
	connect(d_setup_pitch_combo,SIGNAL(currentIndexChanged(int)),this,SLOT(Setup_Changed(void)));
	
	connect(d_graph_mode_button,SIGNAL(toggled(bool)),this,SLOT(Scope_Mode_Changed(bool)));
}

TEncoderSimulation::~TEncoderSimulation(void)
{
}

void TEncoderSimulation::Gain_A(
	const double						value)
{
	d_channel_a.Set_Gain(value);
	d_channel_a.Update_Signals();
	
	this->Update_Encoder_Signal();
	this->Update_Interpolator_Output();
}

void TEncoderSimulation::Gain_B(
	const double						value)
{
	d_channel_b.Set_Gain(value);
	d_channel_b.Update_Signals();
	
	this->Update_Encoder_Signal();
	this->Update_Interpolator_Output();
}

void TEncoderSimulation::Offset_A(
	const double						value)
{
	d_channel_a.Set_Offset(value);
	d_channel_a.Update_Signals();
	
	this->Update_Encoder_Signal();
	this->Update_Interpolator_Output();
}

void TEncoderSimulation::Offset_B(
	const double						value)
{
	d_channel_b.Set_Offset(value);
	d_channel_b.Update_Signals();
	
	this->Update_Encoder_Signal();
	this->Update_Interpolator_Output();
}

void TEncoderSimulation::Phase_A(
	const double						value)
{
	d_channel_a.Set_Phase(value);
	d_channel_a.Update_Signals();
	
	this->Update_Encoder_Signal();
	this->Update_Interpolator_Output();
}

void TEncoderSimulation::Phase_B(
	const double						value)
{
	d_channel_b.Set_Phase(value);
	d_channel_b.Update_Signals();
	
	this->Update_Encoder_Signal();
	this->Update_Interpolator_Output();
}

void TEncoderSimulation::Setup_Changed(void)
{
	this->Update_Interpolator_Output();
}

void TEncoderSimulation::Scope_Mode_Changed(
	bool								state)
{
	if(state)
	{
		d_graph_mode_button->setIcon(QIcon(":/gui/gui_xy.png"));
		d_signal_graph_widget->LockAspectRatio(true);
		
		d_signal_graph_widget->VerticalLegend(QStringLiteral("Channel B (COS)"));
		d_signal_graph_widget->HorizontalLegend(QStringLiteral("Channel A (SIN)"));
		d_signal_graph_widget->Title(QStringLiteral("Oscilloscope - XY Mode"));
	}
	else
	{
		d_graph_mode_button->setIcon(QIcon(":/gui/gui_ty.png"));
		d_signal_graph_widget->LockAspectRatio(false);
		
		d_signal_graph_widget->VerticalLegend(QStringLiteral("Signal"));
		d_signal_graph_widget->HorizontalLegend(QStringLiteral("Channel AB Radian Position"));
		d_signal_graph_widget->Title(QStringLiteral("Oscilloscope - Dual Channel Mode"));
	}
	
	this->Update_Encoder_Signal();
}

void TEncoderSimulation::Update_Encoder_Signal(void)
{
	int									set;
	std::vector<TVector2>				data;
	std::vector<TVector2>				data2;
	std::vector<TVector2>::iterator		iter;
	std::vector<TVector2>::iterator		iter2;
	TVector2							pnt;
	
	d_signal_graph_widget->DeleteDataSets();
	
	if(d_graph_mode_button->isChecked())
	{
		data = d_channel_a.Signal_Data();
		data2 = d_channel_b.Signal_Data();
	
		if(data.size() == data2.size())
		{
			set = d_signal_graph_widget->CreateDataSet(QColor(0,0,128));
			
			for(iter = data.begin(),iter2 = data2.begin();iter != data.end();++iter,++iter2)
			{
				pnt.x = (*iter).y;
				pnt.y = (*iter2).y;
				
				d_signal_graph_widget->AddPoint(set,pnt);
			}
		}
	}
	else
	{
		data = d_channel_a.Signal_Data();
		
		if(data.size())
		{
			set = d_signal_graph_widget->CreateDataSet(QColor(128,0,0));

			for(iter = data.begin();iter != data.end();++iter)
			{
				d_signal_graph_widget->AddPoint(set,(*iter));
			}
		}
		
		data = d_channel_b.Signal_Data();
		
		if(data.size())
		{
			set = d_signal_graph_widget->CreateDataSet(QColor(0,128,0));
			
			for(iter = data.begin();iter != data.end();++iter)
			{
				d_signal_graph_widget->AddPoint(set,(*iter));
			}
		}
	}

	d_signal_graph_widget->UpdateGraph();
}

void TEncoderSimulation::Update_Interpolator_Output(void)
{
	int									set;
	std::vector<TVector2>				data;
	std::vector<TVector2>::iterator		iter;
	int									interpolator_count;
	int									cntr;
	bool								error;
	double								scale_pitch;
	double								scale_resolution;
	QTextCursor 						text_cursor;
	int 								text_cursor_position;
	int 								scroll_bar_vpos;
	int 								scroll_bar_hpos;

	text_cursor_position = d_output_text_edit->textCursor().position();
	scroll_bar_vpos = d_output_text_edit->verticalScrollBar()->value();
	scroll_bar_hpos = d_output_text_edit->horizontalScrollBar()->value();

	scale_pitch = d_setup_pitch_combo->currentText().toDouble();
	scale_resolution = d_setup_resolution_combo->currentText().toDouble();
	
	d_output_text_edit->clear();
	
	if((scale_pitch > 0) == false || (scale_resolution > 0.0) == false)
	{
		return;
	}
	
	interpolator_count = static_cast<int>(0.5 + scale_pitch / scale_resolution);
	
	if(interpolator_count < 2)
	{
		return;
	}
	
	d_interpolator.Set_Interpolator_Count(interpolator_count);
	data = d_interpolator.Interpolate(scale_pitch,d_channel_a,d_channel_b,&error);

	d_output_graph_widget->DeleteDataSets();

	if(data.size())
	{
		d_output_text_edit->appendPlainText(QString("Scale Pitch:        %1").arg(scale_pitch,0,'f',4));
		d_output_text_edit->appendPlainText(QString("Scale Resolution:   %1").arg(scale_resolution,0,'f',4));
		d_output_text_edit->appendPlainText(QString("Interpolator Count: %1\n").arg(interpolator_count));

		d_output_text_edit->appendPlainText(QStringLiteral("Point#    Position   Interpolation Error"));
		d_output_text_edit->appendPlainText(QStringLiteral("----------------------------------------\n"));
		
		set = d_output_graph_widget->CreateDataSet(QColor(0,0,128));
		
		for(iter = data.begin(),cntr = 1;iter != data.end();++iter,++cntr)
		{
			d_output_text_edit->appendPlainText(QString("%1 %2 %3").arg(cntr,5,10).arg((*iter).x,10,'f',4).arg((*iter).y,10,'f',4));
			d_output_graph_widget->AddPoint(set,(*iter));
		}
		
		text_cursor = d_output_text_edit->textCursor();
		text_cursor.setPosition(text_cursor_position);
		
		d_output_text_edit->setTextCursor(text_cursor);
		d_output_text_edit->verticalScrollBar()->setValue(scroll_bar_vpos);
		d_output_text_edit->horizontalScrollBar()->setValue(scroll_bar_hpos);

	}
	
	if(error)
	{
		d_output_status_label->setText(QStringLiteral("Scale count not zero on return"));
	}
	else
	{
		d_output_status_label->setText(QString());
	}
	
	d_output_graph_widget->UpdateGraph();
}









